# Error Message Generator 2.0

This is a modern remake of my "Error Message Generator" that was originally written using Perl/Tk around 2006. This one is written in Go in 2023 using the [Fyne UI](https://fyne.io/) and aims to match and surpass the features of the original Perl version.

![Screenshot](screenshot.png)

The _original_ inspiration for this back in 2006 was an online web tool called "Atom Smasher's Error Message Generator" which generated images of error messages. My Perl ErrorGen program included all of the Atom Smasher icons. Atom Smasher's page seems offline recently, so interest has skyrocketed for my Perl ErrorGen program, so I decided to dust it off, redo it in Go and make it better than ever before.

# Features

This program has two operating modes:

* MainWindow mode: by default, it will show the Main Window where you can construct and test a custom error message pop-up.
* [Command Line Interface](#command-line-interface) (CLI): If given command-line options, it will _just_ pop up the message box you specified and print the user's selection to its standard output, useful for batch files or shell scripts to ask the user a question.

The current features include:

* Choose from 72 built-in icons that all came from the original Atom Smasher's Error Message Generator.
* Custom title bar and message text.
* Configure up to three buttons (via the MainWindow; unlimited number of buttons by command line) each with custom labels.
* Mark some buttons disabled and not clickable.
* Command line interface to use these error dialogs from scripts and external programs:
   * The program returns the selected button text on its standard output.
   * The `--cancel` value can be printed to standard out if the user X's out of the window without selecting a button.
   * Specify which button should be the primary/default response of the Return key.
   * Specify which button should be the cancel/escape key response.

The roadmap of future features include:

* Improve the command-line interface to bring it up to par with other dialog tools such as Zenity.
   * e.g., add File Open/Save Dialogs, Color Picker Dialogs, and others that can be extremely useful to third-party programs.
   * The graphical Error Message Generator tool will probably not call these much - the primary GUI for this program is very much a fun toy. :)
* In the MainWindow, add a "Save" button that will turn your custom error dialog _into_ a batch file or shell script that calls the ErrorGen program's command line interface.
   * Example: you use the ErrorGen UI to create a prank dialog box and "Save" it as a .bat file on disk.
   * Making sure that the .bat and ErrorGen.exe are in the same folder together, double-clicking on the .bat script will show your error dialog without the MainWindow coming up!
* Allow users to select a custom icon image.
   * The command-line interface already supports this.
   * The MainWindow should allow users to browse and select a custom icon from their computer to use in their toy error boxes.
   * Allow users to permanently extend the icon library with a folder of their own customs, so they don't have to browse and select a custom file every single time.

# Command Line Interface

You can call this program from batch files or shell scripts to prompt the user with a graphical dialog box and read the button they chose from this program's standard output.

For example, from a bash script:

```bash
#!/bin/bash
answer=`./errorgen --alert --title "Disk Failure" \
    --text "Failure to read from drive Z:/" \
    --icon defrag --button Abort --button Retry --button Fail \
    --default 1 --cancel Fail`
echo "You have chosen: $answer"
```

![Screenshot from the above command](error.png)

The full documentation is included below. Running `errorgen --help` from
a text terminal will have the program print its documentation to you, but
this won't work on the Windows version since a program must decide if it's
a graphical or a command line app and can't be both.

# Built-in Icons

These are all the icons that currently come with ErrorGen:

| Image | Name |
|-------|------|
![aim_guy](icons/aim_guy.png) | aim_guy |
![aol_icon](icons/aol_icon.png)  | aol_icon |
![attention](icons/attention.png)   | attention |
![bomb](icons/bomb.png) | bomb |
![bomb_dynamite](icons/bomb_dynamite.png) | bomb_dynamite |
![bomb_grenade](icons/bomb_grenade.png)   | bomb_grenade |
![bubble_i](icons/bubble_i.png)  | bubble_i |
![bubble_q](icons/bubble_q.png)  | bubble_q |
![bulb](icons/bulb.png) | bulb |
![butterfly](icons/butterfly.png)   | butterfly |
![cake](icons/cake.png) | cake |
![circularsaw](icons/circularsaw.png)  | circularsaw |
![control_panel](icons/control_panel.png) | control_panel |
![cow](icons/cow.png)   | cow |
![defrag](icons/defrag.png)   | defrag |
![disk_blu](icons/disk_blu.png)  | disk_blu |
![disk_blu_lbl](icons/disk_blu_lbl.png)   | disk_blu_lbl |
![disk_org](icons/disk_org.png)  | disk_org |
![disk_red](icons/disk_red.png)  | disk_red |
![disk_red_lbl](icons/disk_red_lbl.png)   | disk_red_lbl |
![disk_skull](icons/disk_skull.png) | disk_skull |
![disk_yel](icons/disk_yel.png)  | disk_yel |
![dos](icons/dos.png)   | dos |
![e_orbit](icons/e_orbit.png) | e_orbit |
![error](icons/error.png)  | error |
![error2](icons/error2.png)   | error2 |
![error3](icons/error3.png)   | error3 |
![error4](icons/error4.png)   | error4 |
![file_cabinet](icons/file_cabinet.png)   | file_cabinet |
![find](icons/find.png) | find |
![fortunecookie](icons/fortunecookie.png) | fortunecookie |
![garbage_empty](icons/garbage_empty.png) | garbage_empty |
![garbage_full](icons/garbage_full.png)   | garbage_full |
![gun](icons/gun.png)   | gun |
![hammer](icons/hammer.png)   | hammer |
![heart](icons/heart.png)  | heart |
![help](icons/help.png) | help |
![hub](icons/hub.png)   | hub |
![hwinfo](icons/hwinfo.png)   | hwinfo |
![ic_a](icons/ic_a.png) | ic_a |
![keys](icons/keys.png) | keys |
![keys2](icons/keys2.png)  | keys2 |
![keys3](icons/keys3.png)  | keys3 |
![labtec](icons/labtec.png)   | labtec |
![mac](icons/mac.png)   | mac |
![mail](icons/mail.png) | mail |
![mail_deleted](icons/mail_deleted.png)   | mail_deleted |
![mailbox](icons/mailbox.png) | mailbox |
![mouth](icons/mouth.png)  | mouth |
![mycomputer](icons/mycomputer.png) | mycomputer |
![mycomputer2](icons/mycomputer2.png)  | mycomputer2 |
![mycomputer3](icons/mycomputer3.png)  | mycomputer3 |
![newspaper](icons/newspaper.png)   | newspaper |
![peripheral](icons/peripheral.png) | peripheral |
![plant_leaf](icons/plant_leaf.png) | plant_leaf |
![radiation](icons/radiation.png)   | radiation |
![ram](icons/ram.png)   | ram |
![recycle](icons/recycle.png) | recycle |
![recycle2](icons/recycle2.png)  | recycle2 |
![scanner](icons/scanner.png) | scanner |
![screw](icons/screw.png)  | screw |
![screw2](icons/screw2.png)   | screw2 |
![setup](icons/setup.png)  | setup |
![sknife](icons/sknife.png)   | sknife |
![skull](icons/skull.png)  | skull |
![skull2](icons/skull2.png)   | skull2 |
![skull3](icons/skull3.png)   | skull3 |
![tux](icons/tux.png)   | tux |
![tux_config](icons/tux_config.png) | tux_config |
![ups](icons/ups.png)   | ups |
![zipdisk](icons/zipdisk.png) | zipdisk |
![zipdisks](icons/zipdisks.png)  | zipdisks |

# Command Line Usage

The result of `errorgen --help`:

```
NAME:
   errorgen - make fun and custom error messages.

USAGE:
   errorgen [global options] command [command options] [arguments...]

AUTHOR:
   Noah Petherbridge, https://www.kirsle.net

COMMANDS:
   help, h  Shows a list of commands or help for one command

GLOBAL OPTIONS:
   --version, -v  Show program version and build information and exit. (default: false)
   --error, -e    Make the dialog box be of type 'error' by default.

      An info box has these default attributes unless overridden by
      other options:

      Title: Error
      Text: An error has occurred.
      Icon: error
      Buttons: Ok (default: false)
   --alert, -a  Make the dialog box be of type 'alert' by default.

      An alert box has these default attributes unless overridden by
    other options:

      Title: Warning
      Text: Are you sure you want to proceed?
      Icon: attention
      Buttons: Ok (default: false)
   --info, -i  Make the dialog box be of type 'info' by default.

      An info box has these default attributes unless overridden by
    other options:

      Title: Information
      Text: All updates are complete.
      Icon: info
      Buttons: Ok (default: false)
   --question, -q  Make the dialog box be of type 'question' by default.

      A question box has these default attributes unless overridden by
    other options:

      Title: Question
      Text: Are you sure you want to proceed?
      Icon: question
      Buttons: Ok, Cancel (default: false)
   --title value, -n value  Set the title for the dialog box.

      If no title is set, a default is used based on the type of dialog
      box requested (which is --info by default; see also --error,
      --info, --question)
   --text value, -t value  The text displayed inside the dialog box.

      Use \n where you want line breaks.

      Important: the text does NOT have automatic word wrapping applied.
      Insert your own line breaks or else the dialog can be VERY wide for
      longer messages!
   --button value, -b value  Add a button to the dialog.

      Use this option as many times as you want to add multiple buttons.
      The one the user chooses will be printed to standard output. If no
      buttons, default is an Ok button.  (accepts multiple inputs)
   --default value, -m value             Set the default button by its label.

      The matching --button will be made the default/primary button.
   --cancel value, -c value  Set the 'cancel button'.

      This value should match one of your --buttons and will be the
      value 'selected' if the user closes the window or hits the
      Escape key.
   --disabled value, -d value, -g value  Mark buttons (by index) disabled

      Specify a --button number (from 0 to n) that should be disabled,
      or greyed out. The user can not click on this button.  (accepts multiple inputs)
   --icon value, -o value                                    Set the icon to be displayed in the dialog box.

      Use either a built-in icon by name (e.g. "aim_guy") or name your
      own custom PNG image from disk.

      Built-in icon names include the following:

          aim_guy
          aol_icon
          attention
          bomb
          bomb_dynamite
          bomb_grenade
          bubble_i
          bubble_q
          bulb
          butterfly
          cake
          circularsaw
          control_panel
          cow
          defrag
          disk_blu
          disk_blu_lbl
          disk_org
          disk_red
          disk_red_lbl
          disk_skull
          disk_yel
          dos
          e_orbit
          error
          error2
          error3
          error4
          file_cabinet
          find
          fortunecookie
          garbage_empty
          garbage_full
          gun
          hammer
          heart
          help
          hub
          hwinfo
          ic_a
          keys
          keys2
          keys3
          labtec
          mac
          mail
          mail_deleted
          mailbox
          mouth
          mycomputer
          mycomputer2
          mycomputer3
          newspaper
          peripheral
          plant_leaf
          radiation
          ram
          recycle
          recycle2
          scanner
          screw
          screw2
          setup
          sknife
          skull
          skull2
          skull3
          tux
          tux_config
          ups
          zipdisk
          zipdisks
   --help, -h  show help (default: false)
```

# Building

Requirements:

* Go: https://golang.org
* Fyne dependencies: https://developer.fyne.io/started/

Fedora: `sudo dnf install golang gcc libXcursor-devel libXrandr-devel mesa-libGL-devel libXi-devel libXinerama-devel libXxf86vm-devel`

Ubuntu/Debian: `sudo apt-get install golang gcc libgl1-mesa-dev xorg-dev`

There is a GNU Makefile to make build commands easier to remember.

* `make setup` installs Go dependencies for this project.
* `make` or `make build` will produce the ./errorgen binary.
* `make run` runs the Go app locally for development and testing.]
* `make clean` removes the built binary.

Cross compiling: install Docker and fyne-cross and run `make release` to produce the build. To install fyne-cross:

```bash
go install github.com/fyne-io/fyne-cross
```

In case of permission errors (Docker often needs run as root) you can try sudo running fyne-cross directly, e.g.
to produce Windows amd64 and i386 builds a command like:

```bash
# which fyne-cross to use your locally installed Go binary for the current user,
# so that root doesn't need to also install fyne-cross and have a $PATH set up.
sudo $(which fyne-cross) windows -arch=\*
sudo $(which fyne-cross) linux -arch=\*

# Fix ownership permissions on the output folder
sudo chown -R user:user fyne-cross
```

# License & Author

Icons from the Atom Smasher Error Message Generator.

Developed by Noah Petherbridge, <https://www.kirsle.net/>

See my web page about this at <https://www.kirsle.net/errorgen>

    Error Message Generator
    Copyright (C) 2023  Noah Petherbridge

    This program is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with this program.  If not, see <http://www.gnu.org/licenses/>.